# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from datetime import datetime
from report import report_sxw
import pooler

class Overdue(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(Overdue, self).__init__(cr, uid, name, context=context)
        self.localcontext.update( {
            'time': time,
            'adr_get': self._adr_get,
            'getLines': self._lines_get,
            'tel_get': self._tel_get,
            'message': self._message,
        })
        self.context = context
    def _adr_get(self, partner, type):
        res = []
        res_partner = pooler.get_pool(self.cr.dbname).get('res.partner')
        res_partner_address = pooler.get_pool(self.cr.dbname).get('res.partner.address')
        addresses = res_partner.address_get(self.cr, self.uid, [partner.id], [type])
        adr_id = addresses and addresses[type] or False
        result = {
                  'name': False,
                  'street': False,
                  'street2': False,
                  'city': False,
                  'zip': False,
                  'state_id':False,
                  'country_id': False,
                 }
        if adr_id:
            result = res_partner_address.read(self.cr, self.uid, [adr_id], context=self.context.copy())
            result[0]['country_id'] = result[0]['country_id'] and result[0]['country_id'][1] or False
            result[0]['state_id'] = result[0]['state_id'] and result[0]['state_id'][1] or False
            return result

        res.append(result)
        return res

    def _tel_get(self,partner):
        if not partner:
            return False
        res_partner_address = pooler.get_pool(self.cr.dbname).get('res.partner.address')
        res_partner = pooler.get_pool(self.cr.dbname).get('res.partner')
        addresses = res_partner.address_get(self.cr, self.uid, [partner.id], ['invoice'])
        adr_id = addresses and addresses['invoice'] or False
        if adr_id:
            adr=res_partner_address.read(self.cr, self.uid, [adr_id])[0]
            return adr['phone']
        else:
            return partner.address and partner.address[0].phone or False
        return False
    
    def _lines_get(self, partner):
        result = []
        balance_overdue = 0.0
        balance_paid = 0.0
        moveline_obj = pooler.get_pool(self.cr.dbname).get('account.move.line')
        invoice_obj = pooler.get_pool(self.cr.dbname).get('account.invoice')
        movelines = []
        if self.name == 'tko.account.overdue':
            movelines = moveline_obj.search(self.cr, self.uid,
                [('partner_id', '=', partner.id),
                    ('account_id.type', 'in', ['receivable', 'payable']),
                    ('state', '<>', 'draft'), ('reconcile_id', '=', False),('journal_id.type','not in',['cash', 'bank'])], order = 'date')
        elif self.name == 'account.receipts.payments':
            movelines = moveline_obj.search(self.cr, self.uid,
                [('partner_id', '=', partner.id),
                    ('account_id.type', 'in', ['receivable', 'payable']),
                    ('state', '<>', 'draft')], order = 'date')
        for id in movelines:
            res = {}
            moveline = moveline_obj.browse(self.cr, self.uid, id)
            invoice_id = invoice_obj.search(self.cr, self.uid, [('move_id','=',moveline.move_id.id)])
            res['residual'] = 0.0
            if moveline.journal_id.type in ('sale_refund', 'purchase_refund'):
                res['type'] = 'N/Nt. Crédito'
            elif moveline.journal_id.type in ('sale', 'purchase'):
                    res['type'] = 'N/Factura'
            elif moveline.journal_id.type in ('general'):
                 res['type'] = 'N/Nt. Débito'
            elif moveline.journal_id.type in ('cash', 'bank'):
                 res['type'] = 'N/Recibo'
            if moveline.move_id.name == '/':
                res['name_move'] = moveline.name
            else:
                res['name_move'] = moveline.move_id.name
            res['date'] = moveline.date 
            if moveline.date_maturity:
                res['date_ma'] = moveline.date_maturity
                date = datetime.strptime(moveline.date_maturity,'%Y-%m-%d').date()
                now = datetime.today().date()
                n_days = (now - date).days
                if n_days > 0 and moveline.journal_id.type not in ('cash', 'bank'):
                    res['nr_days'] = n_days
                else:
                    res['nr_days'] = '-'
            else:
                res['nr_days'] = '-'
                res['date_ma'] = '-'
            res['total'] = moveline.debit - moveline.credit
            res['debit'] = moveline.debit
            res['credit'] = moveline.credit
            balance_paid += res['debit'] + (res['credit'] * -1)
            res['balance_paid'] = balance_paid
            if invoice_id:
                invoice = invoice_obj.browse(self.cr, self.uid, invoice_id)[0]
                res['pays']= " "
                if invoice.type == 'money_sale':
                    res['type'] = 'N/V.Dinheiro'
                for id in invoice.payment_ids:
                    ref = id.move_id.name
                    if len(invoice.payment_ids)>1:
                         res['pays'] += ref + ', '
                    else:
                        res['pays'] = ref
                if moveline.journal_id.type in ('sale_refund', 'purchase_refund'):
                    res['residual'] = invoice.residual * -1
                else:
                    res['residual'] = invoice.residual
                res['paid'] = (moveline.debit - moveline.credit) - res['residual']
                if invoice.type not in ('in_invoice','money_sale', 'in_refund','in_debit_note') and self.name == 'tko.account.overdue':
                    balance_overdue += res['residual']
                    res['balance'] = balance_overdue
                    result.append(res)
                elif self.name == 'account.receipts.payments':
                    result.append(res)
            elif moveline.journal_id.type in ('cash', 'bank') and self.name == 'account.receipts.payments':
                result.append(res)
        return result

    def _message(self, obj, company):
        company_pool = pooler.get_pool(self.cr.dbname).get('res.company')
        message = company_pool.browse(self.cr, self.uid, company.id, {'lang':obj.lang}).overdue_msg
        return message

report_sxw.report_sxw('report.tko.account.overdue', 'res.partner',
        'tko_account_pt/report/account_print_overdue.rml', parser=Overdue)
report_sxw.report_sxw('report.account.receipts.payments', 'res.partner',
        'tko_account_pt/report/account_receipts_payments.rml', parser=Overdue)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

